'****************************************************************
'*  Name    : ConnectEVE_HardSPI_V3.BAS                         *
'*  Author  : Bobby Garrett                                     *
'*  Notice  : Copyright (c) 2014                                *
'*          : All Rights Reserved                               *
'*  Date    : 08/09/2014                                        *
'*  Version : 3.0                                               *
'*  Notes   : Connect EVE trial converted from FTDI AN 275      *
'*          : and FT800.h                                       *
'*          : Numeric and text data added.                      *
'*          * Clock & Gauge widgets added.                      * 
'****************************************************************

' V3 Added lines and box routine 

' V2 Bitmaps added with assistance from Bob Recny (FTDI)

' V1 First draft from AN275


        Include "amicus18.inc"
        All_Digital = true

        Include "FT800_1_2.INC"	' FT800 register, memory and command values
        Include "SPI.inc"        
        
        Clear                   ' Clear ALL RAM locations
        
        Declare Stack_Size = 40 ' Just for passing parameters to subroutines (Ensure that stack is larger than the 
                                ' amount passed into the sub EG ftGauge = 20 so 40 should be enough.
                                ' The compiler will not complain but the Widget etc will not work correctly.
        
'****************************************************************          
'* Set serial comms parameters                                  *
'* Program does not necessarily use serial comms, it is here so *
'* I can send data out to Hyperterminal for debugging.          *
'****************************************************************   

' Not needed as in Amicus18.inc
 
    '    Declare Hserial_Baud = 9600	     ' Set baud rate to 9600 (20MHz XTAL)
    '    Declare Hserial_RCSTA = %10010000   ' Enable serial port and continuous receive
    '    Declare Hserial_TXSTA = %00100100   ' Enable transmit and asynchronous mode 
    '    Declare Hserial_Clear = On		     ' Enable Error clearing on received characters  
                
' Set LCD display resolution here (Only select one!!)
''#define LCD_QVGA			               	    ' QVGA  = 320 x 240 (VM800B/C 3.5")
#define LCD_WQVGA			                   	' WQVGA = 480 x 272 (VM800B/C 4.3" and 5.0")

' FT800 Chip Commands - use with cmdWrite
        Symbol FT800_ACTIVE     = 0x00			' Initializes FT800
        Symbol FT800_STANDBY	= 0x41			' Place FT800 in Standby (clk running)
        Symbol FT800_SLEEP  	= 0x42			' Place FT800 in Sleep (clk off)
        Symbol FT800_PWRDOWN	= 0x50			' Place FT800 in Power Down (core off)
        Symbol FT800_CLKEXT	    = 0x44			' Select external clock source
        Symbol FT800_CLK48M	    = 0x62			' Select 48MHz PLL
        Symbol FT800_CLK36M	    = 0x61			' Select 36MHz PLL
        Symbol FT800_CORERST	= 0x68			' Reset core - all registers default

' FT800 Memory Commands - use with ft800memWritexx and ft800memReadxx
        Symbol MEM_WRITE    	= 0x80			' FT800 Host Memory Write 
        Symbol MEM_READ     	= 0x00			' FT800 Host Memory Read

' Colours - fully saturated colours defined here
        Symbol RED	         	= 0xFF0000      ' Red
        Symbol GREEN    		= 0x00FF00      ' Green
        Symbol BLUE	        	= 0x0000FF      ' Blue
        Symbol WHITE	       	= 0xFFFFFF      ' White
        Symbol BLACK    		= 0x000000      ' Black

        CCP1CON = 0                   ' Turn off Capture/PWM etc
        CCP2CON = 0                   ' Turn off Capture/PWM etc

' ConnectEVE    Amicus18
' Pin1 CS   --- PORTC.2 
' Pin2 SCK  --- PORTC.3
' Pin3 MISO --- PORTC.4
' Pin4 MOSI --- PORTC.5
' Pin5 INT  --- PORTB.0
' Pin6 PD   --- PORTC.1
' Pin7 Audio + 
' Pin8 Audio -
' Pin9 3.3v
' Pin10 Gnd
        
        Symbol ft800irqPin = PORTB.0  ' FT800 Interrupt Output (not used in this example)
        Symbol ft800pwrPin = PORTC.1  ' PD_N from Arduino to FT800 - effectively FT800 reset
        
' SPI BitOrder(MsbFirst)              ' Send data most significant bit first
' SPI DataMode(SPI_MODE0)             ' Clock idle zero, clock data into FT800 on rising edge       
        Symbol ft800csPin = PORTC.2   ' SPI chip Select - defined separately since it's manipulated with GPIO calls
        Symbol SCK = PORTC.3          ' SPI Clock pin
        Symbol SO = PORTC.4           ' SPI data IN from SO on Connect EVE
        Symbol SI = PORTC.5           ' SPI data OUT to SI on Connect EVE

'Set port direction 
        TRISB.0 = 1       
        TRISC.1 = 0
        TRISC.2 = 0
        TRISC.3 = 0
        TRISC.4 = 1
        TRISC.5 = 0

' LCD display parameters
        Dim lcdWidth As Word          ' Active width of LCD display
        Dim lcdHeight As Word         ' Active height of LCD display
        Dim lcdHcycle As Word         ' Total number of clocks per line
        Dim lcdHoffset As Word        ' Start of active line
        Dim lcdHsync0 As Word         ' Start of horizontal sync pulse
        Dim lcdHsync1 As Word         ' End of horizontal sync pulse
        Dim lcdVcycle As Word         ' Total number of lines per screen
        Dim lcdVoffset As Word        ' Start of active screen
        Dim lcdVsync0 As Word         ' Start of vertical sync pulse
        Dim lcdVsync1 As Word         ' End of vertical sync pulse
        Dim lcdPclk As Byte           ' Pixel Clock
        Dim lcdSwizzle As Byte        ' Define RGB output pins
        Dim lcdPclkpol As Byte        ' Define active edge of PCLK

        Dim ramDisplayList As Dword   ' Set beginning of display list memory 
        ramDisplayList = RAM_DL
        
        Dim ramCommandBuffer As Dword ' Set beginning of graphics command memory
        ramCommandBuffer = RAM_CMD
        
        Dim cmdBufferRd As Word       ' Used to navigate command ring buffer
        Dim cmdBufferWr As Word       ' Used to navigate command ring buffer
        Dim cmdOffset As Word         ' Used to navigate command rung buffer
        
' Although point_size is a Dword it's max value can only be 8191.
' It has to be this size as it is "OR" ed with a Dword in the main loop.
        Dim point_size As Dword        
        point_size = 0x00000100       ' Define a default dot size

'Position the coloured dot to the top right        
        Dim point_x As Dword          ' (320 * 16) Defines the x-location (1/16 anti-aliased)
        point_x = (320 * 16)
        Dim point_y As Dword          ' (54 * 16) Defines the y-location (1/16 anti-aliased)
        point_y = (54 * 16)

'Position the FTDI logo to the top right
        Dim Logo_point_x As Dword          
        Logo_point_x = (385 * 16)     ' (385 * 16) Defines the x-location (1/16 anti-aliased)
        Dim Logo_point_y As Dword      
        Logo_point_y = (40 * 16)      ' (40 * 16) Defines the y-location (1/16 anti-aliased)
       
        Dim colour As Dword           ' Variable for changing colours
        Dim ft800Gpio As Byte         ' Used for FT800 GPIO register
        
        Dim newOffset As Word         ' Used To hold new offset
        
        Dim ftAddress As Dword
        Dim ftData8 As Byte
        Dim ftData16 As Word
        Dim ftData32 As Dword
        
        Dim currentOffset As Word
        Dim commandSize As Byte
        Dim ftCommand As Byte
        
        Dim ReturnedByte As Byte      ' Used where a returned byte from a sub is used in a conditional statement

        Dim BL_duty As Byte           ' Backlight duty
        
' These variables are used as there was an issue sending values (from symbols) to sub-routines if the variable
' was a Word or DWord and the symbol value was less than a Word/Dword.
' The issue was when the value was "popped" from the stack.
' !!!! ftAddressTemp may not be needed.       
        Dim ftAddressTemp As Dword
        Dim memWrite8Data As Byte     ' Might not be needed (used in loading data to RAM_G area)        
        Dim memWrite16Data As Word
        Dim memWrite32Data As Dword  
        
        Dim X_POS As Word
        Dim Y_POS As Word
        Dim FT_FONT As Word
        Dim FT_OPTIONS As Word 
        Dim FT_NUMBER As SDword       'Note - sDword is 2s compliment
        Dim FT_RADIUS As Word
        Dim FT_HOURS As Word
        Dim FT_MINUTES As Word
        Dim FT_SECONDS As Word        
        Dim FT_MILLS As Word  
        
        Dim FT_MAJOR As Word
        Dim FT_MINOR As Word        
        Dim FT_VALUE As Word
        Dim FT_RANGE As Word        
        
        Dim FT_TEXT As string*100     ' Set to 100 just for testing
        Dim FTArray[100] As Byte      ' Create a 50-byte array.
        Dim TextCount As Byte
        Dim TextOffset As Byte
        Dim TextOffsetMod As Byte
        
        Dim PosNr As Dword           ' Positive number used for testing positive numbers
        Dim NegNr As SDword          ' Negative number used for testing negative numbers

        Dim BM_StartLoc As Dword     ' Used for loading the bitmap data to the ft800
        Dim BM_DataLoc As Dword      ' Used for loading the bitmap data to the ft800
        Dim BM_DataLen As Dword      ' Used for loading the bitmap data to the ft800 
        Dim Img_Cnt As Dword         ' Loop counter in Bitmap load sub routine
              
'New for lines
        Dim LineColour As Dword     
        Dim LineStart_X_POS As Dword
        Dim LineStart_Y_POS As Dword
        Dim LineEnd_X_POS As Dword
        Dim LineEnd_Y_POS As Dword 
        Dim LineWidth As Dword  
                
        Dim LineCol As Dword       
        Dim Start_X_POS_AL As Dword        
        Dim Start_Y_POS_AL As Dword          
        Dim End_X_POS_AL As Dword         
        Dim End_Y_POS_AL As Dword 
        Dim LineW As Dword      
        
'New for box
        Dim BOX_TOP As Dword
        Dim BOX_RIGHT As Dword        
        Dim BOX_BOT As Dword
        Dim BOX_LEFT As Dword  
        
        Dim BOX_TOP_AL As Dword
        Dim BOX_RIGHT_AL As Dword        
        Dim BOX_BOT_AL As Dword
        Dim BOX_LEFT_AL As Dword                     
                            

        OpenSPI(SPI_FOSC_16, MODE_00, SMPEND) 'Hardware SPI definitions for Amicus18
        
        GoTo SetUp_FT800             ' Jump over sub routines 

'***************************************************************
' * Function:        ft800memWrite8[ftAddress, ftData8]        *
' * Input:           ftAddress = FT800 memory space address    *
' *                  ftData8 = Byte to send                    *
' * Output:          None                                      *
' * Overview:        Writes FT800 internal address space       *
'***************************************************************
ft800memWrite8:

Pop     ftData8   ' Pop the last variable pushed
Pop     ftAddress ' Pop the first variable pushed

        ftAddress.Byte2 = ftAddress.Byte2 | MEM_WRITE 'Logically OR MEM_WRITE with high address byte.

        ft800csPin = 0   
        
        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte
        
        WriteSPI(ftData8)                          ' Send data Byte

        ft800csPin = 1                             ' Set CS pin High
        Return

'***************************************************************
' * Function:        ft800memWrite16[ftAddress, ftData16]      *
' * PreCondition:    None                                      *
' * Input:           ftAddress = FT800 memory space address    *
' *                  ftData16 = Int to send                    *
' * Output:          None                                      *
' * Side Effects:    None                                      *
' * Overview:        Writes FT800 internal address space       *
'***************************************************************
ft800memWrite16:

Pop     ftData16  ' Pop the last variable pushed
Pop     ftAddress ' Pop the first variable pushed

        ftAddress.Byte2 = ftAddress.Byte2 | MEM_WRITE 'Logically OR MEM_WRITE with high address byte.
      
        ft800csPin = 0                             ' Set CS pin Low

        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte
        
        WriteSPI(ftData16.Byte0)                   ' Send Low data Byte
        WriteSPI(ftData16.Byte1)                   ' Send high data Byte

        ft800csPin = 1                             ' Set CS pin High
        Return

'***************************************************************
' * Function:        ft800memWrite32[ftAddress, ftData32]      *
' * PreCondition:    None                                      *
' * Input:           ftAddress = FT800 memory space address    *
' *                  ftData32 = Dword to send                  *
' * Output:          None                                      *
' * Side Effects:    None                                      *
' * Overview:        Writes FT800 internal address space       *
'***************************************************************
ft800memWrite32:

Pop     ftData32  ' Pop the last variable pushed
Pop     ftAddress ' Pop the first variable pushed

        ftAddress.Byte2 = ftAddress.Byte2 | MEM_WRITE 'Logically OR MEM_WRITE with high address byte.

        ft800csPin = 0                             ' Set CS pin Low
        
        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte
        
        WriteSPI(ftData32.Byte0)                   ' Send Low data Byte
        WriteSPI(ftData32.Byte1)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte2)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte3)                   ' Send high data Byte

        ft800csPin = 1                             ' Set CS pin High
        Return

'***************************************************************
' * Function:        ft800memRead8[ftAddress]                  *
' * Input:           ftAddress = FT800 memory space address    *
' * Output:          Returns data to ftData8                   *
' * Overview:        Reads from FT800 internal address space   *
'***************************************************************
ft800memRead8:

Pop     ftAddress ' Pop the first variable pushed  
  
        ftAddress.Byte2 = ftAddress.Byte2 | MEM_READ 'Logically OR MEM_READ with high address byte.  

        ft800csPin = 0                              ' Set CS pin low

        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte

        WriteSPI(ZERO)                             ' Send a dummy byte 
        
        ftData8.Byte0 = ReadSPI()                 ' Read the data byte in

        ft800csPin = 1                             ' Set CS pin High
        Return ftData8 

'***************************************************************
' * Function:        ft800memRead16[ftAddress]                 *
' * Input:           ftAddress = FT800 memory space address    *
' * Output:          Returns data to ftData16                  *
' * Overview:        Reads from FT800 internal address space   *
'***************************************************************
ft800memRead16:

Pop     ftAddress ' Pop the first variable pushed 

        ftAddress.Byte2 = ftAddress.Byte2 | MEM_READ 'Logically OR MEM_READ with high address byte.  
        
        ft800csPin = 0                             ' Set CS pin low
        
        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte

        WriteSPI(ZERO)                             ' Send a dummy byte 
        
        ftData16.Byte0 = ReadSPI()                 ' Read the low data byte in
        ftData16.Byte1 = ReadSPI()                 ' Read the high data byte in 

        ft800csPin = 1                             ' Set CS pin High
        Return ftData16 

'***************************************************************
' * Function:        ft800memRead32[ftAddress]                 *
' * Input:           ftAddress = FT800 memory space address    *
' * Output:          Returns data to ftData32                  *
' * Overview:        Reads from FT800 internal address space   *
'***************************************************************
ft800memRead32:

Pop     ftAddress ' Pop the first variable pushed 

        ftAddress.Byte2 = ftAddress.Byte2 | MEM_READ 'Logically OR MEM_READ with high address byte.  
        
        ft800csPin = 0                             ' Set CS pin low

        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte

        WriteSPI(ZERO)                             ' Send a dummy byte 
        
        ftData32.Byte0 = ReadSPI()                 ' Read the low data byte in
        ftData32.Byte1 = ReadSPI()                 ' Read the middle data byte in 
        ftData32.Byte2 = ReadSPI()                 ' Read the middle data byte in 
        ftData32.Byte3 = ReadSPI()                 ' Read the high data byte in  
          
        ft800csPin = 1                             ' Set CS pin High
        Return ftData32 

' **************************************************************
' * Function:        ft800cmdWrite[ftCommand]                  *
' * Input:           None                                      *
' * Output:          None                                      *
' * Overview:        Sends FT800 command                       *
' * Note:            None                                      *
' **************************************************************
ft800cmdWrite:

Pop     ftCommand ' Pop the first variable pushed 
        ft800csPin = 0                             ' Set CS pin Low
        
        WriteSPI (ftCommand)                       ' Send command
        WriteSPI (0x00)                     
        WriteSPI (0x00)                 
 
        ft800csPin = 1                             ' Set CS pin High
        Return

' ******************************************************************
' * Function:        incCMDOffset[cmdOffset, 4],cmdOffset          *
' *                  starting a command list                       *
' * Input:           currentOffset = graphics processor command list pointer
' *                  commandSize = number of bytes To increment the offset
' * Output:          newOffset = the new ring buffer pointer after adding the command
' * Overview:        Adds commandSize To the currentOffset.        *
' *                  Checks For 4K ring-buffer offset roll-over    *
' * Note:            None                                          *
' ******************************************************************
incCMDOffset:     ' used To hold new offset

Pop commandSize   ' Pop the last variable pushed
Pop currentOffset ' Pop the first variable pushed 

    newOffset = currentOffset + commandSize     ' Calculate new offset
    If newOffset > 4095 Then         			' If new offset past boundary...
    newOffset = newOffset - 4096                ' ... roll over pointer
    EndIf
    Return newOffset                            ' Return new offset value

' *************************************************************************************************
' * Function:        CmdText[ftAddressTemp, ftData32, X_POS, Y_POS, FT_FONT, FT_OPTIONS, FT_TEXT] *
' * Note:            Text is limited to 100 chrs as per FT_TEXT & FTArray                         *
' * Note:            Command pointer is updated in the subroutine                                 *
' ************************************************************************************************* 
CmdText:

' Pop from stack in reverse order        
        Pop FTArray
        Pop FT_OPTIONS
        Pop FT_FONT         
        Pop Y_POS         
        Pop X_POS         
        Pop ftData32
        Pop ftAddress   

        ftAddress.Byte2 = ftAddress.Byte2 | MEM_WRITE 'Logically OR MEM_WRITE with high address byte.

        ft800csPin = 0                             ' Set CS pin Low
        
        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte
        
        WriteSPI(ftData32.Byte0)                   ' Send Low data Byte
        WriteSPI(ftData32.Byte1)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte2)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte3)                   ' Send high data Byte
        
        WriteSPI(X_POS.Byte0)                      ' Send Low data Byte
        WriteSPI(X_POS.Byte1)                      ' Send High data Byte
        
        WriteSPI(Y_POS.Byte0)                      ' Send low data Byte
        WriteSPI(Y_POS.Byte1)                      ' Send High data Byte
        
        WriteSPI(FT_FONT.Byte0)                    ' Send low data Byte
        WriteSPI(FT_FONT.Byte1)                    ' Send High data Byte
        
        WriteSPI(FT_OPTIONS.Byte0)                 ' Send low data Byte
        WriteSPI(FT_OPTIONS.Byte1)                 ' Send High data Byte

'Loop through the text array until 0 (NULL) is detected        
        TextCount = 0
        While FTArray[TextCount] != 0 
        WriteSPI(FTArray [TextCount])              ' Send Byte         
        Inc TextCount
        Wend
        
        WriteSPI(ZERO)                             ' Send NULL
        
        ft800csPin = 1                             ' Set CS pin High

'Calculate the value to pad out the cmdOffset as it has to be a multiple of 4
        TextOffset = TextCount + 13      '13 is derived from 12 fixed bytes and 1 as TextCount starts at 0 
        TextOffsetMod = TextOffset // 4  'Get the Modulus
       
        If TextOffsetMod = 1 Then TextOffset = TextOffset + 3 
        If TextOffsetMod = 2 Then TextOffset = TextOffset + 2         
        If TextOffsetMod = 3 Then TextOffset = TextOffset + 1     
     
        GoSub incCMDOffset[cmdOffset, TextOffset],cmdOffset      ' Update the command pointer 
        Return

' *****************************************************************************************************
' * Function:        CmdNumber[ftAddressTemp, ftData32, X_POS, Y_POS, FT_FONT, FT_OPTIONS, FT_NUMBER] *
' * Note:            Command pointer is updated in the subroutine                                     *
' * Note:            Ensure that the number is a (S)Dword or it is moved into one before the sub call *
' ***************************************************************************************************** 
CmdNumber:

' Pop from stack in reverse order
        Pop FT_NUMBER 
        Pop FT_OPTIONS        
        Pop FT_FONT         
        Pop Y_POS         
        Pop X_POS         
        Pop ftData32 
        Pop ftAddress   

' Will need to modify this so that any options passed in IE OPT_CENTER are "OR"d with the sign bit. 


' Check to see if number is negative and alter the option as required
        If FT_NUMBER.31 = 1 Then
        FT_OPTIONS = OPT_SIGNED
        Else FT_OPTIONS = 0
        EndIf
                 
        ftAddress.Byte2 = ftAddress.Byte2 | MEM_WRITE 'Logically OR MEM_WRITE with high address byte.

        ft800csPin = 0                             ' Set CS pin Low
        
        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte
        
        WriteSPI(ftData32.Byte0)                   ' Send Low data Byte
        WriteSPI(ftData32.Byte1)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte2)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte3)                   ' Send high data Byte
        
        WriteSPI(X_POS.Byte0)                      ' Send Low data Byte
        WriteSPI(X_POS.Byte1)                      ' Send High data Byte
        
        WriteSPI(Y_POS.Byte0)                      ' Send low data Byte
        WriteSPI(Y_POS.Byte1)                      ' Send High data Byte
        
        WriteSPI(FT_FONT.Byte0)                    ' Send low data Byte
        WriteSPI(FT_FONT.Byte1)                    ' Send High data Byte
        
        WriteSPI(FT_OPTIONS.Byte0)                 ' Send low data Byte
        WriteSPI(FT_OPTIONS.Byte1)                 ' Send High data Byte

        WriteSPI(FT_NUMBER.Byte0)                  ' Send low data Byte
        WriteSPI(FT_NUMBER.Byte1)                  ' Send middle data Byte
        WriteSPI(FT_NUMBER.Byte2)                  ' Send middle data Byte
        WriteSPI(FT_NUMBER.Byte3)                  ' Send High data Byte  

        ft800csPin = 1                             ' Set CS pin High
        
        GoSub incCMDOffset[cmdOffset, 16],cmdOffset      ' Update the command pointer  
        Return

' *****************************************************************************************************
' * Function:        CmdGauge[ftAddressTemp, ftData32, X_POS, Y_POS, FT_RADIUS, FT_OPTIONS, FT_MAJOR, *
'                    FT_MINOR, FT_VALUE, FT_RANGE]                                                    *
' * Note:            Command pointer is updated in the subroutine                                     *
' ***************************************************************************************************** 
CmdGauge:

' Pop from stack in reverse order
        Pop FT_RANGE   
        Pop FT_VALUE                 
        Pop FT_MINOR  
        Pop FT_MAJOR         
        Pop FT_OPTIONS         
        Pop FT_RADIUS 
        Pop Y_POS         
        Pop X_POS         
        Pop ftData32                                
        Pop ftAddress  
       
        ftAddress.Byte2 = ftAddress.Byte2 | MEM_WRITE 'Logically OR MEM_WRITE with high address byte.

        ft800csPin = 0                             ' Set CS pin Low
        
        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte
        
        WriteSPI(ftData32.Byte0)                   ' Send Low data Byte
        WriteSPI(ftData32.Byte1)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte2)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte3)                   ' Send high data Byte
        
        WriteSPI(X_POS.Byte0)                      ' Send Low data Byte
        WriteSPI(X_POS.Byte1)                      ' Send High data Byte
        
        WriteSPI(Y_POS.Byte0)                      ' Send low data Byte
        WriteSPI(Y_POS.Byte1)                      ' Send High data Byte
        
        WriteSPI(FT_RADIUS.Byte0)                  ' Send low data Byte
        WriteSPI(FT_RADIUS.Byte1)                  ' Send High data Byte
        
        WriteSPI(FT_OPTIONS.Byte0)                 ' Send low data Byte
        WriteSPI(FT_OPTIONS.Byte1)                 ' Send High data Byte
       
        WriteSPI(FT_MAJOR.Byte0)                   ' Send low data Byte
        WriteSPI(FT_MAJOR.Byte1)                   ' Send High data Byte
        
        WriteSPI(FT_MINOR.Byte0)                   ' Send low data Byte
        WriteSPI(FT_MINOR.Byte1)                   ' Send High data Byte
        
        WriteSPI(FT_VALUE.Byte0)                   ' Send low data Byte
        WriteSPI(FT_VALUE.Byte1)                   ' Send High data Byte
        
        WriteSPI(FT_RANGE.Byte0)                   ' Send low data Byte
        WriteSPI(FT_RANGE.Byte1)                   ' Send High data Byte                
                 
        ft800csPin = 1                             ' Set CS pin High
        
        GoSub incCMDOffset[cmdOffset, 20],cmdOffset      ' Update the command pointer   
        Return

' *****************************************************************************************************
' * Function:        CmdClock[ftAddressTemp, ftData32, X_POS, Y_POS, FT_RADIUS, FT_OPTIONS, FT_HOURS, *
'                    FT_MINUTES, FT_SECONDS, FT_MILLS]                                                *
' * Note:            Command pointer is updated in the subroutine                                     *
' ***************************************************************************************************** 
CmdClock:

' Pop from stack in reverse order
        Pop FT_MILLS   
        Pop FT_SECONDS              
        Pop FT_MINUTES         
        Pop FT_HOURS         
        Pop FT_OPTIONS 
        Pop FT_RADIUS 
        Pop Y_POS         
        Pop X_POS         
        Pop ftData32        
        Pop ftAddress    

        ftAddress.Byte2 = ftAddress.Byte2 | MEM_WRITE 'Logically OR MEM_WRITE with high address byte.

        ft800csPin = 0                             ' Set CS pin Low
        
        WriteSPI(ftAddress.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(ftAddress.Byte1)                  ' Send middle address Byte  
        WriteSPI(ftAddress.Byte0)                  ' Send Low address Byte
        
        WriteSPI(ftData32.Byte0)                   ' Send Low data Byte
        WriteSPI(ftData32.Byte1)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte2)                   ' Send middle data Byte
        WriteSPI(ftData32.Byte3)                   ' Send high data Byte
        
        WriteSPI(X_POS.Byte0)                      ' Send Low data Byte
        WriteSPI(X_POS.Byte1)                      ' Send High data Byte
        
        WriteSPI(Y_POS.Byte0)                      ' Send low data Byte
        WriteSPI(Y_POS.Byte1)                      ' Send High data Byte
        
        WriteSPI(FT_RADIUS.Byte0)                  ' Send low data Byte
        WriteSPI(FT_RADIUS.Byte1)                  ' Send High data Byte
        
        WriteSPI(FT_OPTIONS.Byte0)                 ' Send low data Byte
        WriteSPI(FT_OPTIONS.Byte1)                 ' Send High data Byte
        
        WriteSPI(FT_HOURS.Byte0)                   ' Send low data Byte
        WriteSPI(FT_HOURS.Byte1)                   ' Send High data Byte
        
        WriteSPI(FT_MINUTES.Byte0)                 ' Send low data Byte
        WriteSPI(FT_MINUTES.Byte1)                 ' Send High data Byte
        
        WriteSPI(FT_SECONDS.Byte0)                 ' Send low data Byte
        WriteSPI(FT_SECONDS.Byte1)                 ' Send High data Byte
        
        WriteSPI(FT_MILLS.Byte0)                   ' Send low data Byte
        WriteSPI(FT_MILLS.Byte1)                   ' Send High data Byte
                 
        ft800csPin = 1                             ' Set CS pin High
        
        GoSub incCMDOffset[cmdOffset, 20],cmdOffset      ' Update the command pointer  
        Return
        
' *****************************************************************************************************************
' * Function:        CmdDrawLine[LineColour,LineStart_X_POS,LineStart_Y_POS,LineEnd_X_POS,LineEnd_X_POS,LineWidth]*
' * Note:            Command pointer is updated in the subroutine                                                 *
' ***************************************************************************************************************** 
CmdDrawLine:

' Pop from stack in reverse order
        Pop LineW
        Pop End_Y_POS_AL        
        Pop End_X_POS_AL         
        Pop Start_Y_POS_AL         
        Pop Start_X_POS_AL         
        Pop LineCol 

'Defines the x/y-locations (1/16 anti-aliased)        
        Start_X_POS_AL = (Start_X_POS_AL * 16)
        Start_Y_POS_AL = (Start_Y_POS_AL * 16)  
        End_X_POS_AL = (End_X_POS_AL * 16)
        End_Y_POS_AL = (End_Y_POS_AL * 16)
        LineW = (LineW * 16)

' Set the colour of the line
        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_COLOUR_RGB | LineCol) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_BEGIN | LINES) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (Start_X_POS_AL << 15) | Start_Y_POS_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (End_X_POS_AL << 15) | End_Y_POS_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_LINE_WIDTH | LineW) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = DL_END 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' End the line

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer 

        Return

' ***************************************************************************************
' * Function:        CmdDrawBox[LineColour,BOX_TOP,BOX_RIGHT,BOX_BOT,BOX_LEFT,LineWidth]*
' * Note:            Command pointer is updated in the subroutine                       *
' *************************************************************************************** 
' There should be a "RECTS" option for "BEGIN" but I could not find anything in the programmers 
' guide so I have written this routine.

CmdDrawBox:

' Pop from stack in reverse order
        Pop LineW
        Pop BOX_LEFT_AL        
        Pop BOX_BOT_AL         
        Pop BOX_RIGHT_AL         
        Pop BOX_TOP_AL         
        Pop LineCol 

'Defines the x/y-locations (1/16 anti-aliased)        
        BOX_TOP_AL = (BOX_TOP_AL * 16)
        BOX_RIGHT_AL = (BOX_RIGHT_AL * 16)  
        BOX_BOT_AL = (BOX_BOT_AL * 16)
        BOX_LEFT_AL = (BOX_LEFT_AL * 16)
        LineW = (LineW * 16)

' Set the colour of the box
        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_COLOUR_RGB | LineCol) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_BEGIN | LINES) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

'Draw the top line
        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (BOX_LEFT_AL << 15) | BOX_TOP_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

' Set the line width, this will be set for all the lines now.        
        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_LINE_WIDTH | LineW) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer        

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (BOX_RIGHT_AL << 15) | BOX_TOP_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] 

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

'Draw the Right line
        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (BOX_RIGHT_AL << 15) | BOX_TOP_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (BOX_RIGHT_AL << 15) | BOX_BOT_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] 
        
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer
        
'Draw the Bottom line
        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (BOX_LEFT_AL << 15) | BOX_BOT_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (BOX_RIGHT_AL << 15) | BOX_BOT_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]         

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer
        
'Draw the Left line
        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (BOX_LEFT_AL << 15) | BOX_BOT_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]   

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (BOX_LEFT_AL << 15) | BOX_TOP_AL)  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]               

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = DL_END 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' End the line

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset ' Update the command pointer 

        Return
        
' *****************************************************************************************************
' * Function:        BitMapLoad[BM_StartLoc, BM_DataLoc, BM_DataLen]                                  *
' *****************************************************************************************************         
BitMapLoad:

' Pop from stack in reverse order
        Pop BM_DataLen                             ' Length of the bitmap data
        Pop BM_DataLoc                             ' Location of the bitmap data
        Pop BM_StartLoc                            ' Start location where the bitmap data is to be stored

' Set the start location for the bitmap
        Img_Cnt = BM_StartLoc
        
' Load the first data byte        
        memWrite8Data = LRead BM_DataLoc + Img_Cnt 
                 
'Write the bitmap data to the FT800
        BM_StartLoc.Byte2 = BM_StartLoc.Byte2 | MEM_WRITE 'Logically OR MEM_WRITE with high address byte.

        ft800csPin = 0   
        
        WriteSPI(BM_StartLoc.Byte2)                  ' Send Memory Write plus High address Byte
        WriteSPI(BM_StartLoc.Byte1)                  ' Send middle address Byte  
        WriteSPI(BM_StartLoc.Byte0)                  ' Send Low address Byte
                      
While   Img_Cnt < BM_DataLen                         ' Loop until end of data.

        WriteSPI(memWrite8Data)                      ' Send data Byte
        Inc Img_Cnt  
        memWrite8Data = LRead BM_DataLoc + Img_Cnt   ' Get the next data byte
Wend
        ft800csPin = 1                               ' Set CS pin High
        Return

' *****************************************************************************
' * Function:        Setup                                                    *
' * Overview:        System initialization routines.  Executed once On Reset. *
' *****************************************************************************
  
SetUp_FT800:

' LCD display parameters
#ifdef LCD_QVGA			              ' QVGA display parameters
        lcdWidth   = 320              ' Active width of LCD display
        lcdHeight  = 240              ' Active height of LCD display
        lcdHcycle  = 408              ' Total number of clocks per line
        lcdHoffset = 70               ' Start of active line
        lcdHsync0  = 0                ' Start of horizontal sync pulse
        lcdHsync1  = 10               ' End of horizontal sync pulse
        lcdVcycle  = 263              ' Total number of lines per screen
        lcdVoffset = 13               ' Start of active screen
        lcdVsync0  = 0                ' Start of vertical sync pulse
        lcdVsync1  = 2                ' End of vertical sync pulse
        lcdPclk    = 8                ' Pixel Clock
        lcdSwizzle = 2                ' Define RGB output pins
        lcdPclkpol = 0                ' Define active edge of PCLK
#endif

#ifdef LCD_WQVGA                      ' WQVGA display parameters
        lcdWidth   = 480              ' Active width of LCD display
        lcdHeight  = 272              ' Active height of LCD display
        lcdHcycle  = 548              ' Total number of clocks per line
        lcdHoffset = 43               ' Start of active line
        lcdHsync0  = 0                ' Start of horizontal sync pulse
        lcdHsync1  = 41               ' End of horizontal sync pulse
        lcdVcycle  = 292              ' Total number of lines per screen
        lcdVoffset = 12               ' Start of active screen
        lcdVsync0  = 0                ' Start of vertical sync pulse
        lcdVsync1  = 10               ' End of vertical sync pulse
        lcdPclk    = 5                ' Pixel Clock
        lcdSwizzle = 0                ' Define RGB output pins
        lcdPclkpol = 1                ' Define active edge of PCLK
#endif

        ft800csPin = 1                ' Set CS pin high for start - SPI inactive
        ft800pwrPin = 1               ' Set PD pin high for start
        
        DelayMS 20                    ' Wait a few MS before waking the FT800

'Wake-upFT800:
  
        ft800pwrPin = 0               ' 1) lower PD pin
        DelayMS 20                    ' 2) hold for 20ms
        ft800pwrPin = 1               ' 3) raise PD pin
        DelayMS 20                    ' 4) wait for another 20ms before sending any commands
        
        GoSub ft800cmdWrite[FT800_ACTIVE]     ' Start FT800
        DelayMS 5                             ' Give some time to process
        GoSub ft800cmdWrite[FT800_CLKEXT]     ' Set FT800 for external clock
        DelayMS 5                             ' Give some time to process
        GoSub ft800cmdWrite[FT800_CLK48M]     ' Set FT800 for 48MHz PLL
        DelayMS 5                             ' Give some time to process
    
  			' Now FT800 can accept commands At up To 30MHz clock On SPI bus
  			' This application leaves the SPI bus At 4MHz

        GoSub ft800memRead8[REG_ID],ReturnedByte
        If ReturnedByte != 0x7C	Then          ' Read ID register - is it 0x7C?
        HRSOut "System Halted! ", 10,10, 13   ' Send an error message on the UART
                 ' If we don't get 0x7C, the interface isn't working - halt with infinite loop
        EndIf
  
        GoSub ft800memWrite8[REG_PCLK, ZERO]     ' Set PCLK to zero - don't clock the LCD until later
        GoSub ft800memWrite8[REG_PWM_DUTY, ZERO] ' Turn off backlight
  
' End of Wake-up FT800
'***************************************
        
'***************************************
' Initialize Display

        ftAddressTemp = REG_HSIZE
        memWrite16Data = lcdWidth     
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' active display width

        ftAddressTemp = REG_HCYCLE
        memWrite16Data = lcdHcycle         
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' total number of clocks per line, incl front/back porch
        
        ftAddressTemp = REG_HOFFSET
        memWrite16Data = lcdHoffset          
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' start of active line  

        ftAddressTemp = REG_HSYNC0
        memWrite16Data = lcdHsync0           
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' start of horizontal sync pulse
 
        ftAddressTemp = REG_HSYNC1
        memWrite16Data = lcdHsync1  
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' end of horizontal sync pulse

        ftAddressTemp = REG_VSIZE
        memWrite16Data = lcdHeight          
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' active display height
   
        ftAddressTemp = REG_VCYCLE
        memWrite16Data = lcdVcycle         
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' total number of lines per screen, incl pre/post
        
        ftAddressTemp = REG_VOFFSET
        memWrite16Data = lcdVoffset            
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' start of active screen

        ftAddressTemp = REG_VSYNC0
        memWrite16Data = lcdVsync0         
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' start of vertical sync pulse

        ftAddressTemp = REG_VSYNC1
        memWrite16Data = lcdVsync1            
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data]    ' end of vertical sync pulse
        
        GoSub ft800memWrite8[REG_SWIZZLE, lcdSwizzle]           ' FT800 output to LCD - pin order

        GoSub ft800memWrite8[REG_PCLK_POL, lcdPclkpol]          ' LCD data is clocked in on this PCLK edge

		' Don't set PCLK yet - wait for just after the first display list
        	
' End of Initialize Display
'***************************************

'***************************************
' Configure Touch And Audio - not used in this example, so Disable both

        GoSub ft800memWrite8[REG_TOUCH_MODE, ZERO]      ' Disable touch

        ftAddressTemp = REG_TOUCH_RZTHRESH
        memWrite16Data = ZERO        
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data] ' Eliminate any false touches

        GoSub ft800memWrite8[REG_VOL_PB, ZERO]          ' turn recorded audio volume down
        GoSub ft800memWrite8[REG_VOL_SOUND, ZERO]       ' turn synthesizer volume down
        GoSub ft800memWrite16[REG_SOUND, 0x6000]        ' set synthesizer to mute
  
' End of Configure Touch And Audio
'***************************************

' Load the FTDI image data into RAM_G.  This is fine for a static images

' Write the bitmap data to the FT800
        BM_StartLoc = RAM_G                  ' Load the bitmap at location 0
        BM_DataLoc = VarPtr(LOGO)                        
        BM_DataLen = 1008                    ' Value taken from FTDI conversion file (see LOGO data at end of code)
        
        GoSub BitMapLoad[BM_StartLoc, BM_DataLoc, BM_DataLen] 
  
' The command ring buffer is needed when changing images on the fly.  
' Use CMD_MEMWRITE or CMD_INFLATE inline with the ring buffer.
' With CMD_MEMWRITE or CMD_INFLATE, the REG_CMDREAD and REG_CMDWRITE cannot overlap >> keep  
' updating REG_CMDWRITE and Wait For REG_CMDREAD to catch up so the ring buffer is not overwritten.
  
' NOTE -- ALL DATA in RAM_G needs to start on a 4-byte boundary for each object 
' For this example the FTDI logo starts at 0 and ends at 1007, a font or next bitmap would then start at 1008.  
' If the logo ended at 1013, the font or next bitmap would *still* start at 1016.


'***************************************
' Write Initial Display List & Enable Display

        ramDisplayList = RAM_DL                         ' start of Display List
        
        ftAddressTemp = ramDisplayList
        memWrite32Data = DL_CLEAR_RGB      
        GoSub ft800memWrite32[ramDisplayList, DL_CLEAR_RGB]
        
        ' Clear Color RGB   00000010 RRRRRRRR GGGGGGGG BBBBBBBB  (R/G/B = Colour values) default zero / black
        ramDisplayList = ramDisplayList + 4             ' point to next location

        ftAddressTemp = ramDisplayList
        memWrite32Data = (DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG)     
        GoSub ft800memWrite32[ramDisplayList, memWrite32Data]
        ' Clear 00100110 -------- -------- -----CST  (C/S/T define which parameters to clear)

        ramDisplayList = ramDisplayList + 4             ' point to next location

        ftAddressTemp = ramDisplayList
        memWrite32Data = DL_DISPLAY        
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]
        ' DISPLAY command 00000000 00000000 00000000 00000000 (end of display list)

        ftAddressTemp = REG_DLSWAP
        memWrite32Data = DLSWAP_FRAME
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]
        ' 00000000 00000000 00000000 000000SS  (SS bits define when render occurs)

	    ' Nothing is being displayed yet... the Pixel clock is still 0x00
					
        ramDisplayList = RAM_DL                         ' Reset Display List pointer for next list

        GoSub ft800memRead8[REG_GPIO],ft800Gpio         ' Read the FT800 GPIO register for a read/modify/write operation  
               
        ft800Gpio = ft800Gpio | 0x80                    ' set bit 7 of FT800 GPIO register (DISP) - others are inputs

        GoSub ft800memWrite8[REG_GPIO, ft800Gpio]       ' Enable the DISP signal to the LCD panel
   
        GoSub ft800memWrite8[REG_PCLK, lcdPclk]         ' Now start clocking data to the LCD panel
     
        For BL_duty = 0 To 127 Step 1                   ' Loop for back light duty

        GoSub ft800memWrite8[REG_PWM_DUTY, BL_duty]     ' Turn on backlight - ramp up slowly to full brighness

        DelayMS 10
        
        Next BL_duty

' End of Write Initial Display List & Enable Display
'***************************************

' End of setup

'***************************************************************************
' * Function:        Main program Loop                                     *
' * Overview:        Demonstrate the capsbilities of the FT800/ConnectEVE  *
' *                  Continually change a dot from WHITE To RED And back   *
' *                  Shows Clock & Gauge widgets                           *
' *                  Shows positive and negative numbers                   *
' *                  Shows Text                                            *
'***************************************************************************

Main_Loop:
' Wait For graphics processor To complete executing the current command list
' This happens when REG_CMD_READ matches REG_CMD_WRITE, indicating that all commands
' have been executed.  The Next commands get executed when REG_CMD_WRITE is updated again...
' Then REG_CMD_READ again catches up To REG_CMD_WRITE
' This is a 4Kbyte ring buffer, so KEEP pointers within the 4K roll-over
        
        Repeat
      
        GoSub ft800memRead16[REG_CMD_READ],cmdBufferRd  ' Read the graphics processor read pointer  
        GoSub ft800memRead16[REG_CMD_WRITE],cmdBufferWr ' Read the graphics processor write pointer   
        Until cmdBufferWr = cmdBufferRd                 ' Wait until the two registers match
  
        cmdOffset = cmdBufferWr                         ' The new starting point the first location after the last command

        If colour != WHITE Then				            ' If a RED dot was just drawn (Or first time through)...
        colour = WHITE                                  ' change colour to white
        Else						                    ' Otherwise...
        colour = RED                                    ' change the colour to red
        EndIf

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = CMD_DLSTART              
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' Start the display list  
						                        
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_CLEAR_RGB | BLACK)    
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' Set the default Clear color To BLACK
 					                        	
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer
        
        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG)   
		GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' Clear the screen - this And the previous prevent artifacts between lists
						                        ' Attributes are the colour, stencil And tag buffers

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_COLOUR_RGB | colour) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]
						                        ' Set the colour of the following item(s) - Toggle RED/WHITE from above

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_POINT_SIZE | point_size) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]
						                                ' Select the size of the dot To draw

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_BEGIN | POINTS) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]
						                                ' Indicate To draw a point (dot)

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (point_x << 15) | point_y) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]
        						                        ' Set the point center location

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = DL_END 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' End the point
 
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer
       
'--------------------------------------------------------------------------------------
' Numbers - set colour  for positive number count

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_COLOUR_RGB | GREEN) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' Set the colour 

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer
        
'--------------------------------------------------------------------------------------
' Numbers - send positive count data

' Do not worry about OPT_SIGNED as this will be handled inside the subroutine.

        ftAddressTemp = RAM_CMD + cmdOffset  
        ftData32 = CMD_NUMBER
        X_POS = 20
        Y_POS = 30
        FT_FONT = 31
        FT_OPTIONS = 0
        'FT_NUMBER 

        GoSub CmdNumber[ftAddressTemp, ftData32, X_POS, Y_POS, FT_FONT, FT_OPTIONS, PosNr] 

        ' Command pointer is updated in the subroutine 
        
        Inc PosNr 
        If PosNr >= 32000 Then PosNr = 0
        
'--------------------------------------------------------------------------------------
' Numbers - set colour  for negative number count

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_COLOUR_RGB | WHITE) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' Set the colour 

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer
        
'--------------------------------------------------------------------------------------
' Numbers - send negative count data

' Do not worry about OPT_SIGNED as this will be handled inside the subroutine.

        ftAddressTemp = RAM_CMD + cmdOffset  
        ftData32 = CMD_NUMBER
        X_POS = 150
        Y_POS = 30
        FT_FONT = 31
        FT_OPTIONS = 0
        'FT_NUMBER 

        GoSub CmdNumber[ftAddressTemp, ftData32, X_POS, Y_POS, FT_FONT, FT_OPTIONS, NegNr] 

        ' Command pointer is updated in the subroutine 
        
        Dec NegNr 
        If NegNr <= -32000 Then NegNr = 0
        
'--------------------------------------------------------------------------------------     
'ftGauge

        ftAddressTemp = RAM_CMD + cmdOffset
        ftData32 = CMD_GAUGE
        X_POS = 240
        Y_POS = 140
        FT_RADIUS = 50
        FT_OPTIONS = 0
        FT_MAJOR = 5
        FT_MINOR = 4
        'FT_VALUE 'Value is calculated below       
        FT_RANGE = 100               

        GoSub CmdGauge[ftAddressTemp, ftData32, X_POS, Y_POS, FT_RADIUS, FT_OPTIONS, FT_MAJOR, FT_MINOR, FT_VALUE, FT_RANGE] 

        ' Command pointer is updated in the subroutine 

        Inc FT_VALUE 
        If FT_VALUE >= 100 Then FT_VALUE = 0
        
'--------------------------------------------------------------------------------------          
'ftClock

        ftAddressTemp = RAM_CMD + cmdOffset  
        ftData32 = CMD_CLOCK
        X_POS = 400
        Y_POS = 140
        FT_RADIUS = 50
        FT_OPTIONS = 0
  '     FT_HOURS = 8     'Value is calculated below  
  '     FT_MINUTES = 15  'Value is calculated below  
  '     FT_SECONDS = 0   'Value is calculated below        
        FT_MILLS = 0      
        
        GoSub CmdClock[ftAddressTemp, ftData32, X_POS, Y_POS, FT_RADIUS, FT_OPTIONS, FT_HOURS, FT_MINUTES, FT_SECONDS, FT_MILLS] 

        ' Command pointer is updated in the subroutine 
        
        Inc FT_SECONDS 
        If FT_SECONDS >= 60 Then
        FT_SECONDS = 0
        Inc FT_MINUTES
        EndIf
        
        If FT_MINUTES >= 60 Then
        FT_MINUTES = 0
        Inc FT_HOURS
        EndIf        
 
        If FT_HOURS >= 12 Then FT_HOURS = 0
       
'--------------------------------------------------------------------------------------
' Text - set colour

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_COLOUR_RGB | BLUE) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' Set the colour 

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer
        
'--------------------------------------------------------------------------------------
' Text - Send the data 

        ftAddressTemp = RAM_CMD + cmdOffset  
        ftData32 = CMD_TEXT
        X_POS = 20
        Y_POS = 240
        FT_FONT = 20
        FT_OPTIONS = 0
        FT_TEXT = "This is just to see if the text handling is OK, the text arrays are set to 100 at the moment."

        GoSub CmdText[ftAddressTemp, ftData32, X_POS, Y_POS, FT_FONT, FT_OPTIONS, FT_TEXT] 
      
        ' Command pointer is updated in the subroutine 
          
'--------------------------------------------------------------------------------------       

' Bitmap - set colour for the Logo as it is monochrome

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_COLOUR_RGB | WHITE) 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' Set the colour 

        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

'--------------------------------------------------------------------------------------       
' LOGO display

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_BITMAP_SOURCE | RAM_G) 
                                           ' Point to the beginning of the image - this case is beginning of RAM_G
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]
                                        
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_BITMAP_LAYOUT | (L4 << 19) | (36 << 9) | (28 << 0)) 
                                           ' Values taken from FTDI conversion file (see LOGO data at end of code)
                                           ' Format, stride and height all taken from first line of img_cvt output in rawh
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]  
                                        
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_BITMAP_SIZE | (72 << 9) | (28 << 0)) 
                                           ' Values taken from FTDI conversion file (see LOGO data at end of code)
                                           ' Filter, wrapx and wrapy all = 0, width and height located within 32-bit map
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]  
        
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_BEGIN | 1)  ' BITMAPS primitive = 1
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]    
        
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = (DL_VERTEX2F | (Logo_point_x << 15) | Logo_point_y)  ' Draw the LOGO.
                                                        
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]  
        
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = DL_END                         ' End the bitmap  
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]  
        
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

' End of LOGO display
'--------------------------------------------------------------------------------------       

'--------------------------------------------------------------------------------------
' Lines - Send the data 
' Draw a blue line under the text at the bottom of the display
        LineColour = BLUE
        LineStart_X_POS = 1
        LineStart_Y_POS = 260
        LineEnd_X_POS = 480
        LineEnd_Y_POS = 260  
        LineWidth = 2      

        GoSub CmdDrawLine[LineColour, LineStart_X_POS, LineStart_Y_POS, LineEnd_X_POS, LineEnd_Y_POS, LineWidth] 
      
        ' Command pointer is updated in the subroutine 
'--------------------------------------------------------------------------------------   
'--------------------------------------------------------------------------------------
' BOX - Send the data 

' There should be a "RECTS" option for "BEGIN" but I could not find anything in the programmers 
' guide so I have written this routine so it could change in the future.

' Draw a red box around the flashing dot
        LineColour = RED
        BOX_TOP = 25
        BOX_RIGHT = 350        
        BOX_BOT = 85
        BOX_LEFT = 290
        LineWidth = 1      

        GoSub CmdDrawBox[LineColour, BOX_TOP, BOX_RIGHT, BOX_BOT, BOX_LEFT, LineWidth] 
      
        ' Command pointer is updated in the subroutine 
'--------------------------------------------------------------------------------------  


        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = DL_DISPLAY 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data]
						                                ' Instruct the graphics processor To show the list
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = RAM_CMD + cmdOffset
        memWrite32Data = CMD_SWAP 
        GoSub ft800memWrite32[ftAddressTemp, memWrite32Data] ' Make this list active
        
        GoSub incCMDOffset[cmdOffset, 4],cmdOffset      ' Update the command pointer

        ftAddressTemp = REG_CMD_WRITE
        memWrite16Data = cmdOffset 
        GoSub ft800memWrite16[ftAddressTemp, memWrite16Data] ' Update the ring buffer pointer so the 
                                                        ' graphics processor starts executing

        DelayMS 500                                     ' Wait a half-second to observe the changing colour

        GoTo Main_Loop                                  ' End of Main Loop


'--------------------------------------------------------------------------------------  
' Start of FTDI Logo bitmap data
        
' Contents of ftdilogo.rawh copied here -- L4 format used here to keep size down.

LOGO:
' File properties: ', 'resolution ', 72, 'x', 28, 'format ', 'L4', 'stride ', 36, ' total size ', 1008 
LData 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,_
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,_
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,_
255,253,185,171,239,255,255,255,255,255,255,255,255,255,255,255,_
255,255,255,255,255,255,255,255,255,255,255,255,254,202,152,119,_
154,134,84,75,230,51,51,51,51,159,255,255,255,255,255,197,_
33,145,34,34,37,193,34,73,255,255,81,239,255,255,255,255,_
238,220,186,206,219,170,140,248,51,68,68,68,68,51,191,255,_
255,255,250,0,0,128,0,16,3,192,0,0,15,255,64,239,_
255,255,255,254,220,169,140,219,152,101,223,51,52,51,51,51,_
67,67,55,255,255,255,241,14,255,255,246,15,255,176,159,246,_
1,255,64,239,255,255,255,254,220,203,222,220,186,141,243,51,_
51,51,51,51,51,51,51,111,255,255,240,63,255,255,245,14,_
255,176,143,255,80,159,64,239,255,255,254,237,203,156,220,185,_
134,207,84,69,68,68,68,84,68,68,69,57,255,255,240,63,_
238,255,245,14,255,176,143,255,224,63,64,239,255,255,254,220,_
186,174,203,168,120,249,51,51,51,51,51,51,51,51,51,51,_
239,255,240,0,0,175,245,14,255,176,143,255,240,31,64,239,_
255,255,254,237,220,238,221,203,175,243,68,68,68,68,68,68,_
68,68,68,67,127,255,240,40,118,223,245,14,255,176,143,255,_
240,47,64,239,255,255,237,204,186,237,203,152,143,179,51,51,_
52,51,51,51,67,67,52,51,63,255,240,79,255,255,245,14,_
255,176,143,255,144,111,64,239,255,255,238,221,205,237,220,186,_
207,99,51,51,51,51,51,51,51,51,52,51,59,255,240,63,_
255,255,245,14,255,176,143,253,0,255,64,239,255,255,237,220,_
189,237,203,169,223,68,68,68,68,68,68,68,68,68,52,68,_
56,255,240,63,255,255,245,14,255,176,102,64,11,255,64,239,_
255,254,237,220,174,237,203,168,239,51,51,51,51,51,51,51,_
51,51,52,51,54,255,240,47,255,255,244,14,255,160,64,3,_
223,255,48,239,255,255,238,237,206,238,221,203,255,52,68,68,_
68,68,68,68,68,68,68,68,70,255,254,239,255,255,254,239,_
255,254,238,255,255,255,238,255,255,254,237,204,173,237,203,152,_
239,51,51,51,51,51,51,51,67,51,52,51,54,255,255,255,_
255,255,255,255,255,255,255,255,255,255,255,255,255,255,238,237,_
222,238,221,203,239,68,68,68,68,68,68,68,68,68,68,68,_
72,255,255,254,150,248,223,255,255,255,255,255,255,255,255,255,_
255,255,237,220,188,237,203,168,175,99,67,51,52,52,51,51,_
67,67,52,52,59,255,255,80,52,240,191,255,249,95,255,255,_
255,255,255,255,255,255,238,221,204,254,220,187,175,179,51,51,_
51,51,51,51,51,51,51,51,47,255,246,15,255,241,207,255,_
252,175,255,255,255,255,255,255,255,255,238,237,203,222,220,186,_
141,244,68,69,68,68,68,84,68,68,69,68,127,255,240,223,_
255,241,96,11,249,111,247,0,111,255,255,255,255,255,254,237,_
203,174,220,186,135,250,51,51,51,51,51,51,67,51,52,50,_
239,255,176,255,255,242,46,144,248,79,96,221,35,255,255,255,_
255,255,255,254,237,205,254,221,203,207,84,68,68,68,68,68,_
68,68,68,57,255,255,192,255,255,241,191,240,216,79,13,255,_
240,239,255,255,255,255,255,254,237,185,205,204,168,108,243,51,_
52,52,51,51,67,67,51,111,255,255,240,191,255,241,207,240,_
216,79,15,255,240,207,255,255,255,255,255,255,238,220,189,237,_
220,185,239,67,51,51,51,51,51,51,55,255,255,255,249,11,_
255,241,207,240,216,79,12,255,144,255,255,255,255,255,255,255,_
238,220,185,156,203,152,104,249,51,68,68,68,68,51,191,255,_
255,255,255,144,0,240,191,240,216,63,7,1,12,255,255,255,_
255,255,255,255,255,238,220,186,205,203,169,158,231,35,51,51,_
35,159,255,255,255,255,255,255,254,254,255,254,255,239,15,220,_
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,_
255,254,186,172,239,255,255,255,255,255,255,255,255,255,255,255,_
255,255,14,255,255,255,255,255,255,255,255,255,255,255,255,255,_
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,_
255,255,255,255,255,255,14,255,255,255,255,255,255,255,255,255,_
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,_
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255          
        
' End of FTDI Logo bitmap data
'--------------------------------------------------------------------------------------          
        

